#pragma once

#include <ostream>
#include <concepts>
#include <memory>

namespace ext {

class ostream {
	std::unique_ptr < std::ostream > m_os;

public:
	ostream ( std::basic_streambuf < char > * sb ) : m_os ( std::make_unique < std::ostream > ( sb ) ) {
	}

	virtual ~ostream ( ) noexcept = default;

protected:
	ostream ( ostream && rhs ) noexcept : m_os ( std::move ( rhs.m_os ) ) {
	}

	ostream & operator = ( ostream && rhs ) noexcept {
		m_os = std::move ( rhs.m_os );
		return * this;
	}

public:
	template < class T >
	requires requires { std::declval < std::ostream > ( ) << std::declval < T > ( ); }
	friend ostream & operator << ( ostream & os, const T & val ) {
		static_cast < std::ostream & > ( os ) << val;
		return os;
	}

	friend ostream & operator << ( ostream & os, std::ostream & ( * fn ) ( std::ostream & ) ) {
		static_cast < std::ostream & > ( os ) << fn;
		return os;
	}

protected:
	operator std::ostream & ( ) & {
		return * m_os;
	}

	operator const std::ostream & ( ) const & {
		return * m_os;
	}

public:
	ostream & put ( char ch ) {
		m_os->put ( ch );
		return * this;
	}

	ostream & write ( const char * s, std::streamsize count ) {
		m_os->write ( s, count );
		return * this;
	}

	std::streampos tellp ( ) {
		return m_os->tellp ( );
	}

	ostream & seekp ( std::streampos pos ) {
		m_os->seekp ( pos );
		return * this;
	}

	ostream & seekp ( std::streamoff off, std::ios_base::seekdir dir ) {
		m_os->seekp ( off, dir );
		return * this;
	}

	ostream & flush ( ) {
		m_os->flush ( );
		return * this;
	}

	std::streambuf * rdbuf ( ) const {
		return m_os->rdbuf ( );
	}

	std::streambuf * rdbuf ( std::streambuf * buf ) {
		return m_os->rdbuf ( buf );
	}

	void setstate ( std::ios_base::iostate state ) {
		m_os->setstate ( state );
	}

	void clear ( std::ios_base::iostate state = std::ios_base::goodbit ) {
		m_os->clear ( state );
	}

};

} /* namespace ext */
