#pragma once

#include <alib/ostream>
#include <sstream>

namespace ext {

class ostringstream : public ostream {
public:
	ostringstream ( ) : ostream ( new std::stringbuf ( ) ) {
	}

	ostringstream ( ostringstream && rhs ) noexcept : ostream ( std::move ( rhs ) ) {
		rhs = ostringstream ( );
	}

	~ostringstream ( ) noexcept {
		delete rdbuf ( );
	}

	ostringstream & operator = ( ostringstream && ) noexcept = default;

	void swap ( ostringstream & other ) {
		ostringstream tmp ( std::move ( * this ) );
		* this = std::move ( other );
		other = std::move ( tmp );
	}

	std::string str ( ) const & {
		return rdbuf ( )->str ( );
	}

	void str ( const std::string & str ) {
		return rdbuf ( )->str ( str );
	}

	std::stringbuf * rdbuf ( ) {
		return static_cast < std::stringbuf * > ( ostream::rdbuf ( ) );
	}

	const std::stringbuf * rdbuf ( ) const {
		return static_cast < const std::stringbuf * > ( ostream::rdbuf ( ) );
	}
};

} /* namespace ext */
